@@ -1,10 +1,21 @@
+#![cfg_attr(docsrs, feature(doc_cfg))]
+
+//! # Crazyradio driver for Rust
+//!
+//! This crate aims at providing a Rust API for the [Crazyradio](https://www.bitcraze.io/products/crazyradio-pa/)
+//! USB Dongle.
+//!
+//! Available Cargo features:
+//!  - **shared_radio** enables [SharedCrazyradio] object that allows to share a radio between threads
+//!  - **async** enables async function to create a [Crazyradio] object and use the [SharedCrazyradio]
+//!  - **serde** emables [serde](https://crates.io/crates/serde) serialization/deserialization of the [Channel] struct
+
 #[cfg(feature = "shared_radio")]
 mod shared_radio;
 #[cfg(feature = "shared_radio")]
 pub use crate::shared_radio::SharedCrazyradio;
 
 use core::time::Duration;
-use rusb;
 #[cfg(feature = "serde_support")]
 use serde::{Deserialize, Serialize};
 
@@ -30,7 +41,7 @@ fn find_crazyradio(
             n += 1;
         }
     }
-    return Err(Error::NotFound);
+    Err(Error::NotFound)
 }
 
 fn get_serial<T: rusb::UsbContext>(
@@ -39,9 +50,9 @@ fn get_serial<T: rusb::UsbContext>(
 ) -> Result<String> {
     let languages = handle.read_languages(Duration::from_secs(1))?;
 
-    if languages.len() > 0 {
+    if !languages.is_empty() {
         let serial =
-            handle.read_serial_number_string(languages[0], &device_desc, Duration::from_secs(1))?;
+            handle.read_serial_number_string(languages[0], device_desc, Duration::from_secs(1))?;
         Ok(serial)
     } else {
         Err(Error::NotFound)
@@ -59,7 +70,7 @@ fn list_crazyradio_serials() -> Result<Vec<String>> {
 
             let languages = handle.read_languages(Duration::from_secs(1))?;
 
-            if languages.len() > 0 {
+            if !languages.is_empty() {
                 let serial = handle.read_serial_number_string(
                     languages[0],
                     &device_desc,
@@ -236,7 +247,7 @@ impl Crazyradio {
 
     /// Set the radio channel.
     pub fn set_channel(&mut self, channel: Channel) -> Result<()> {
-        if self.cache_settings == false || self.channel != channel {
+        if !self.cache_settings || self.channel != channel {
             self.device_handle.write_control(
                 0x40,
                 UsbCommand::SetRadioChannel as u8,
@@ -253,7 +264,7 @@ impl Crazyradio {
 
     /// Set the datarate.
     pub fn set_datarate(&mut self, datarate: Datarate) -> Result<()> {
-        if self.cache_settings == false || self.datarate != datarate {
+        if !self.cache_settings || self.datarate != datarate {
             self.device_handle.write_control(
                 0x40,
                 UsbCommand::SetDataRate as u8,
@@ -270,7 +281,7 @@ impl Crazyradio {
 
     /// Set the radio address.
     pub fn set_address(&mut self, address: &[u8; 5]) -> Result<()> {
-        if self.cache_settings == false || self.address != *address {
+        if !self.cache_settings || self.address != *address {
             self.device_handle.write_control(
                 0x40,
                 UsbCommand::SetRadioAddress as u8,
@@ -455,6 +466,57 @@ impl Crazyradio {
     }
 }
 
+/// # Async implementations
+///
+/// Async version of open/getserial functions.
+///
+/// Implemented by launching a thread, calling the sync function and passing the
+/// result back though a channel.
+/// This is not the most efficient implementation but it keeps the lib executor-independent
+/// and these functions are only one-time-call in most programs.
+#[cfg(feature = "async")]
+#[cfg_attr(docsrs, doc(cfg(feature = "async")))]
+impl Crazyradio {
+
+    /// Async vesion of [Crazyradio::open_first()]
+    pub async fn open_first_async() -> Result<Self> {
+        let (tx, rx) = flume::bounded(0);
+
+        std::thread::spawn(move || tx.send(Self::open_first()));
+
+        rx.recv_async().await.unwrap()
+    }
+
+    /// Async vesion of [Crazyradio::open_nth()]
+    pub async fn open_nth_async(nth: usize) -> Result<Self> {
+        let (tx, rx) = flume::bounded(0);
+
+        std::thread::spawn(move || tx.send(Self::open_nth(nth)));
+
+        rx.recv_async().await.unwrap()
+    }
+
+    /// Async vesion of [Crazyradio::open_by_serial()]
+    pub async fn open_by_serial_async(serial: &str) -> Result<Self> {
+        let serial = serial.to_owned();
+
+        let (tx, rx) = flume::bounded(0);
+
+        std::thread::spawn(move || tx.send(Self::open_by_serial(&serial)));
+
+        rx.recv_async().await.unwrap()
+    }
+
+    /// Async vesion of [Crazyradio::list_serials()]
+    pub async fn list_serials_async() -> Result<Vec<String>> {
+        let (tx, rx) = flume::bounded(0);
+
+        std::thread::spawn(move || tx.send(Self::list_serials()));
+
+        rx.recv_async().await.unwrap()
+    }
+}
+
 #[derive(thiserror::Error, Debug, Clone)]
 pub enum Error {
     #[error("Usb Error: {0}:?")]
@@ -473,6 +535,7 @@ impl From<rusb::Error> for Error {
     }
 }
 
+/// Ack status of a sent packet
 #[derive(Debug, Copy, Clone)]
 pub struct Ack {
     /// At true if an ack packet has been received
@@ -485,6 +548,7 @@ pub struct Ack {
     pub length: usize,
 }
 
+/// Radio channel
 #[derive(Debug, Copy, Clone, Eq, PartialEq, Hash)]
 #[cfg_attr(feature = "serde_support", derive(Serialize))]
 pub struct Channel(u8);
@@ -512,12 +576,13 @@ impl Channel {
     }
 }
 
-impl Into<u8> for Channel {
-    fn into(self) -> u8 {
-        self.0
+impl From<Channel> for u8 {
+    fn from(val: Channel) -> Self {
+        val.0
     }
 }
 
+/// Radio datarate
 #[derive(Copy, Clone, PartialEq)]
 pub enum Datarate {
     Dr250K = 0,
@@ -525,6 +590,7 @@ pub enum Datarate {
     Dr2M = 2,
 }
 
+/// Radio power
 pub enum Power {
     Pm18dBm = 0,
     Pm12dBm = 1,
